@@ -172,6 +172,11 @@ pub trait RecordDecorator: io::Write {
         self.reset()
     }
 
+    /// Format a file location
+    fn start_location(&mut self) -> io::Result<()> {
+        self.reset()
+    }
+
     /// Format value
     fn start_separator(&mut self) -> io::Result<()> {
         self.reset()
@@ -216,6 +221,11 @@ impl RecordDecorator for Box<dyn RecordDecorator> {
         (**self).start_value()
     }
 
+    /// Format file location
+    fn start_location(&mut self) -> io::Result<()> {
+        (**self).start_location()
+    }
+
     /// Format value
     fn start_separator(&mut self) -> io::Result<()> {
         (**self).start_separator()
@@ -229,6 +239,7 @@ pub fn print_msg_header(
     fn_timestamp: &dyn ThreadSafeTimestampFn<Output = io::Result<()>>,
     mut rd: &mut dyn RecordDecorator,
     record: &Record,
+    use_file_location: bool,
 ) -> io::Result<bool> {
     rd.start_timestamp()?;
     fn_timestamp(&mut rd)?;
@@ -239,6 +250,17 @@ pub fn print_msg_header(
     rd.start_level()?;
     write!(rd, "{}", record.level().as_short_str())?;
 
+    if use_file_location {
+        rd.start_location()?;
+        write!(
+            rd,
+            "[{}:{}:{}]",
+            record.location().file,
+            record.location().line,
+            record.location().column
+        )?;
+    }
+
     rd.start_whitespace()?;
     write!(rd, " ")?;
 
@@ -250,11 +272,49 @@ pub fn print_msg_header(
 
 // }}}
 
+// {{{ Header Printer
+/// Threadsafe header formatting function type
+///
+/// To satify `slog-rs` thread and unwind safety requirements, the
+/// bounds expressed by this trait need to satisfied for a function
+/// to be used in timestamp formatting.
+pub trait ThreadSafeHeaderFn:
+    Fn(
+        &dyn ThreadSafeTimestampFn<Output = io::Result<()>>,
+        &mut dyn RecordDecorator,
+        &Record,
+        bool,
+    ) -> io::Result<bool>
+    + Send
+    + Sync
+    + UnwindSafe
+    + RefUnwindSafe
+    + 'static
+{
+}
+
+impl<F> ThreadSafeHeaderFn for F
+where
+    F: Fn(
+            &dyn ThreadSafeTimestampFn<Output = io::Result<()>>,
+            &mut dyn RecordDecorator,
+            &Record,
+            bool,
+        ) -> io::Result<bool>
+        + Send
+        + Sync,
+    F: UnwindSafe + RefUnwindSafe + 'static,
+    F: ?Sized,
+{
+}
+
+// }}}
+
 // {{{ Term
 /// Terminal-output formatting `Drain`
 ///
 /// **Note**: logging to `FullFormat` drain is thread-safe, since every
-/// line of output is formatted indecently. However, the underlying
+/// line of output is formatted independently. However, the underlying
 /// IO, needs to be synchronized.
 pub struct FullFormat<D>
 where
@@ -263,6 +323,8 @@ where
     decorator: D,
     fn_timestamp: Box<dyn ThreadSafeTimestampFn<Output = io::Result<()>>>,
     use_original_order: bool,
+    use_file_location: bool,
+    header_printer: Box<dyn ThreadSafeHeaderFn>,
 }
 
 /// Streamer builder
@@ -273,6 +335,8 @@ where
     decorator: D,
     fn_timestamp: Box<dyn ThreadSafeTimestampFn<Output = io::Result<()>>>,
     original_order: bool,
+    file_location: bool,
+    header_printer: Box<dyn ThreadSafeHeaderFn>,
 }
 
 impl<D> FullFormatBuilder<D>
@@ -300,6 +364,12 @@ where
         self
     }
 
+    /// Enable the file location in log in this format [file:line:column]
+    pub fn use_file_location(mut self) -> Self {
+        self.file_location = true;
+        self
+    }
+
     /// Use the original ordering of key-value pairs
     ///
     /// By default, key-values are printed in a reversed order. This option will
@@ -309,12 +379,73 @@ where
         self
     }
 
+    /// Provide a function that print the header
+    ///
+    /// If not used, `slog_term::print_msg_header` will be used.
+    ///
+    /// The header is the part before the log message and key-values. It usually contains the time,
+    /// the log level.
+    ///
+    /// The default function:
+    /// ```compile_fail
+    /// pub fn print_msg_header(
+    ///     fn_timestamp: &dyn ThreadSafeTimestampFn<Output = io::Result<()>>,
+    ///     mut rd: &mut dyn RecordDecorator,
+    ///     record: &Record,
+    ///     use_file_location: bool,
+    /// ) -> io::Result<bool> {
+    ///     rd.start_timestamp()?;
+    ///     fn_timestamp(&mut rd)?;
+    ///
+    ///     rd.start_whitespace()?;
+    ///     write!(rd, " ")?;
+    ///
+    ///     rd.start_level()?;
+    ///     write!(rd, "{}", record.level().as_short_str())?;
+    ///
+    ///     if use_file_location {
+    ///         rd.start_location()?;
+    ///         write!(
+    ///             rd,
+    ///             "[{}:{}:{}]",
+    ///             record.location().file,
+    ///             record.location().line,
+    ///             record.location().column
+    ///         )?;
+    ///     }
+    ///
+    ///     rd.start_whitespace()?;
+    ///     write!(rd, " ")?;
+    ///
+    ///     rd.start_msg()?;
+    ///     let mut count_rd = CountingWriter::new(&mut rd);
+    ///     write!(count_rd, "{}", record.msg())?;
+    ///     Ok(count_rd.count() != 0)
+    /// }
+    /// ```
+    ///
+    /// produces this output:
+    /// ```text
+    /// Oct 19 09:20:37.962 INFO an event log, my_key: my_value
+    /// ```
+    ///
+    /// the `Oct 19 09:20:37.962 INFO` part is the header.
+    pub fn use_custom_header_print<F>(mut self, f: F) -> Self
+    where
+        F: ThreadSafeHeaderFn,
+    {
+        self.header_printer = Box::new(f);
+        self
+    }
+
     /// Build `FullFormat`
     pub fn build(self) -> FullFormat<D> {
         FullFormat {
             decorator: self.decorator,
             fn_timestamp: self.fn_timestamp,
             use_original_order: self.original_order,
+            use_file_location: self.file_location,
+            header_printer: self.header_printer,
         }
     }
 }
@@ -346,6 +477,8 @@ where
             fn_timestamp: Box::new(timestamp_local),
             decorator: d,
             original_order: false,
+            file_location: false,
+            header_printer: Box::new(print_msg_header),
         }
     }
 
@@ -355,8 +488,14 @@ where
         values: &OwnedKVList,
     ) -> io::Result<()> {
         self.decorator.with_record(record, values, |decorator| {
-            let comma_needed =
-                print_msg_header(&*self.fn_timestamp, decorator, record)?;
+            let header_printer = &self.header_printer;
+            let comma_needed = header_printer(
+                &*self.fn_timestamp,
+                decorator,
+                record,
+                self.use_file_location,
+            )?;
+
             {
                 let mut serializer = Serializer::new(
                     decorator,
@@ -397,6 +536,7 @@ where
     decorator: D,
     history: RefCell<Vec<(Vec<u8>, Vec<u8>)>>,
     fn_timestamp: Box<dyn ThreadSafeTimestampFn<Output = io::Result<()>>>,
+    header_printer: Box<dyn ThreadSafeHeaderFn>,
 }
 
 /// Streamer builder
@@ -406,6 +546,7 @@ where
 {
     decorator: D,
     fn_timestamp: Box<dyn ThreadSafeTimestampFn<Output = io::Result<()>>>,
+    header_printer: Box<dyn ThreadSafeHeaderFn>,
 }
 
 impl<D> CompactFormatBuilder<D>
@@ -433,12 +574,24 @@ where
         self
     }
 
+    /// Provide a function that print the header
+    ///
+    /// If not used, `slog_term::print_msg_header` will be used
+    pub fn use_custom_header_print<F>(mut self, f: F) -> Self
+    where
+        F: ThreadSafeHeaderFn,
+    {
+        self.header_printer = Box::new(f);
+        self
+    }
+
     /// Build the streamer
     pub fn build(self) -> CompactFormat<D> {
         CompactFormat {
             decorator: self.decorator,
             fn_timestamp: self.fn_timestamp,
             history: RefCell::new(vec![]),
+            header_printer: self.header_printer,
         }
     }
 }
@@ -469,6 +622,7 @@ where
         CompactFormatBuilder {
             fn_timestamp: Box::new(timestamp_local),
             decorator: d,
+            header_printer: Box::new(print_msg_header),
         }
     }
 
@@ -494,8 +648,10 @@ where
                 write!(decorator, " ")?;
             }
 
+            let header_printer = &self.header_printer;
             let comma_needed =
-                print_msg_header(&*self.fn_timestamp, decorator, record)?;
+                header_printer(&*self.fn_timestamp, decorator, record, false)?;
+
             {
                 let mut serializer =
                     Serializer::new(decorator, comma_needed, false);
@@ -687,7 +843,7 @@ impl<'a> slog::ser::Serializer for Serializer<'a> {
             val.as_serde(),
             &mut serde_json::Serializer::new(&mut writer),
         )
-        .map_err(|e| std::io::Error::from(e))?;
+        .map_err(std::io::Error::from)?;
         let val =
             std::str::from_utf8(&writer).expect("serde JSON is always UTF-8");
         s!(self, key, val);
@@ -725,7 +881,7 @@ impl<'a> CompactFormatSerializer<'a> {
             let (print, trunc, push) =
                 if let Some(prev) = self.history.get_mut(indent) {
                     if *prev != buf {
-                        *prev = mem::replace(&mut buf, (vec![], vec![]));
+                        *prev = mem::take(&mut buf);
                         (true, true, false)
                     } else {
                         (false, false, false)
@@ -735,7 +891,7 @@ impl<'a> CompactFormatSerializer<'a> {
                 };
 
             if push {
-                self.history.push(mem::replace(&mut buf, (vec![], vec![])));
+                self.history.push(mem::take(&mut buf));
             }
 
             if trunc {
@@ -927,21 +1083,37 @@ where
 {
 }
 
-const TIMESTAMP_FORMAT: &str = "%b %d %H:%M:%S%.3f";
+const TIMESTAMP_FORMAT: &[time::format_description::FormatItem] = time::macros::format_description!("[month repr:short] [day] [hour repr:24]:[minute]:[second].[subsecond digits:3]");
 
 /// Default local timezone timestamp function
 ///
 /// The exact format used, is still subject to change.
 pub fn timestamp_local(io: &mut dyn io::Write) -> io::Result<()> {
-    write!(io, "{}", chrono::Local::now().format(TIMESTAMP_FORMAT))
+    let now: time::OffsetDateTime = std::time::SystemTime::now().into();
+    write!(
+        io,
+        "{}",
+        now.format(TIMESTAMP_FORMAT)
+            .map_err(convert_time_fmt_error)?
+    )
 }
 
 /// Default UTC timestamp function
 ///
 /// The exact format used, is still subject to change.
 pub fn timestamp_utc(io: &mut dyn io::Write) -> io::Result<()> {
-    write!(io, "{}", chrono::Utc::now().format(TIMESTAMP_FORMAT))
+    let now = time::OffsetDateTime::now_utc();
+    write!(
+        io,
+        "{}",
+        now.format(TIMESTAMP_FORMAT)
+            .map_err(convert_time_fmt_error)?
+    )
 }
+fn convert_time_fmt_error(cause: time::error::Format) -> io::Error {
+    io::Error::new(io::ErrorKind::Other, cause)
+}
+
 // }}}
 
 // {{{ Plain
@@ -1391,13 +1563,13 @@ impl<'a> RecordDecorator for TermRecordDecorator<'a> {
         if !self.use_color {
             return Ok(());
         }
-        match self.term {
-            &mut AnyTerminal::Stdout {
+        match *self.term {
+            AnyTerminal::Stdout {
                 ref mut term,
                 supports_reset,
                 ..
             } if supports_reset => term.reset(),
-            &mut AnyTerminal::Stderr {
+            AnyTerminal::Stderr {
                 ref mut term,
                 supports_reset,
                 ..
@@ -1412,13 +1584,13 @@ impl<'a> RecordDecorator for TermRecordDecorator<'a> {
             return Ok(());
         }
         let color = TermDecorator::level_to_color(self.level);
-        match self.term {
-            &mut AnyTerminal::Stdout {
+        match *self.term {
+            AnyTerminal::Stdout {
                 ref mut term,
                 supports_color,
                 ..
             } if supports_color => term.fg(color as term::color::Color),
-            &mut AnyTerminal::Stderr {
+            AnyTerminal::Stderr {
                 ref mut term,
                 supports_color,
                 ..
